@@ -13,10 +13,12 @@ from optparse import OptionParser, OptionGroup
 
 from PIL import Image as PImage
 
-__version__ = '0.2.5'
+__version__ = '0.2.6'
+
+
+PADDING_REGEXP = re.compile("^(\d+-?){,3}\d+$")
 
 TRANSPARENT = (255, 255, 255, 0)
-
 CAMELCASE_SEPARATOR = 'camelcase'
 CONFIG_FILENAME = 'sprite.conf'
 ORDERINGS = ['maxside', 'width', 'height', 'area']
@@ -25,25 +27,46 @@ PSEUDO_CLASSES = set(['link', 'visited', 'active', 'hover', 'focus',
                       'first-letter', 'first-line', 'first-child',
                       'before', 'after'])
 
-DEFAULT_SETTINGS = {'padding': '0',
-                    'margin': '0',
-                    'algorithm': 'square',
-                    'ordering': 'maxside',
-                    'namespace': 'sprite',
-                    'crop': False,
-                    'url': '',
-                    'less': False,
-                    'optipng': False,
-                    'html': False,
-                    'ignore_filename_paddings': False,
-                    'png8': False,
-                    'separator': '-',
-                    'global_template': ('%(all_classes)s{background-image:'
-                                        'url(%(sprite_url)s);'
-                                        'background-repeat:no-repeat}\n'),
-                    'each_template': ('%(class_name)s{'
-                                      'background-position:%(x)s %(y)s;'
-                                      'width:%(width)s;height:%(height)s;}\n')}
+DEFAULT_SETTINGS = {
+    'padding': '0',
+    'margin': '0',
+    'algorithm': 'square',
+    'ordering': 'maxside',
+    'namespace': 'sprite',
+    'crop': False,
+    'url': '',
+    'less': False,
+    'optipng': False,
+    'html': False,
+    'ignore_filename_paddings': False,
+    'png8': False,
+    'ratios': '',
+    'retina': False,
+    'separator': '-',
+    'optipngpath': 'optipng',
+    'optipng': False,
+    'project': False,
+    'quiet': False,
+    'cachebuster': False,
+    'cachebuster-filename': False,
+    'global_template':
+        ('%(all_classes)s{background-image:url(%(sprite_url)s);'
+         'background-repeat:no-repeat}\n'),
+    'each_template':
+        ('%(class_name)s{background-position:%(x)s %(y)s;'
+         'width:%(width)s;height:%(height)s;}\n'),
+    'ratio_template':
+        ('@media '
+         'only screen and (-webkit-min-device-pixel-ratio: %(ratio)s), '
+         'only screen and (min--moz-device-pixel-ratio: %(ratio)s), '
+         'only screen and (-o-min-device-pixel-ratio: %(ratio_fraction)s), '
+         'only screen and (min-device-pixel-ratio: %(ratio)s) {'
+         '%(all_classes)s{background-image:url(%(sprite_url)s);'
+         '-webkit-background-size: %(width)s %(height)s;'
+         '-moz-background-size: %(width)s %(height)s;'
+         'background-size: %(width)s %(height)s;'
+         '}}\n')
+    }
 
 TEST_HTML_TEMPLATE = """
 <html><head><title>Glue Sprite Test Html</title>
@@ -90,6 +113,61 @@ class PILUnavailableError(Exception):
     error_code = 7
 
 
+class _Missing(object):
+    """ Missing object necessary for cached_property"""
+    def __repr__(self):
+        return 'no value'
+
+    def __reduce__(self):
+        return '_missing'
+
+_missing = _Missing()
+
+
+class cached_property(object):
+    """
+    Decorator inspired/copied from mitsuhiko/werkzeug.
+
+    A decorator that converts a function into a lazy property.  The
+    function wrapped is called the first time to retrieve the result
+    and then that calculated result is used the next time you access
+    the value"""
+
+    def __init__(self, func, name=None, doc=None):
+        self.__name__ = name or func.__name__
+        self.__module__ = func.__module__
+        self.__doc__ = doc or func.__doc__
+        self.func = func
+
+    def __get__(self, obj, type=None):
+        if obj is None:
+            return self
+        value = obj.__dict__.get(self.__name__, _missing)
+        if value is _missing:
+            value = self.func(obj)
+            obj.__dict__[self.__name__] = value
+        return value
+
+
+def round_up(value):
+    int_value = int(value)
+    return int_value + 1 if value != int_value else int_value
+
+
+def nearest_fration(value):
+    """
+    Return the nearest fraction.
+    If fraction.Fraction is not available, return a fraction.
+
+    Note: used for Opera CSS pixel-ratio media queries.
+    """
+    try:
+        from fraction import Fraction
+        return str(Fraction(value))
+    except ImportError:
+        return '%i/100' % int(float(value) * 100)
+
+
 class SquareAlgorithmNode(object):
 
     def __init__(self, x=0, y=0, width=0, height=0, used=False,
@@ -333,8 +411,15 @@ class Image(object):
 
         self.width, self.height = self.image.size
         margin = int(self.sprite.config.margin)
-        self.absolute_width = self.width + self.padding[1] + self.padding[3] + (margin * 2)
-        self.absolute_height = self.height + self.padding[0] + self.padding[2] + (margin * 2)
+
+        # Both absolute_width and absolute_height are the real absolute space
+        # that we need to allocate into the sprite for this image.
+        # It's calculated using the width/height, padding and margin
+        # of this image multiplied by the max_ratio.
+        self.absolute_width = round_up(self.width +
+                (self.horizontal_padding + 2 * margin) * self.sprite.max_ratio)
+        self.absolute_height = round_up(self.height +
+                (self.vertical_padding + 2 * margin) * self.sprite.max_ratio)
 
     def _crop_image(self):
         """Crop the image searching for the smallest possible bounding box
@@ -382,7 +467,7 @@ class Image(object):
             padding = [DEFAULT_SETTINGS['padding']] * 4
         return map(int, padding)
 
-    @property
+    @cached_property
     def class_name(self):
         """Return the CSS class name for this file.
 
@@ -432,15 +517,15 @@ class Image(object):
 
         return separator.join([self.sprite.namespace, name])
 
-    @property
+    @cached_property
     def _padding_info(self):
         """Return the padding information from the filename."""
         for block in self.filename.split('_')[::-1]:
-            if re.match(r"^(\d+-?){,3}\d+$", block):
-                return block.split('-')
+            if PADDING_REGEXP.match(block):
+                return  block.split('-')
         return []
 
-    @property
+    @cached_property
     def padding(self):
         """Return the padding for this image based on the filename and
         the sprite settings file.
@@ -457,6 +542,14 @@ class Image(object):
             padding = self.sprite.config.padding
         return self._generate_padding(padding)
 
+    @cached_property
+    def horizontal_padding(self):
+        return self.padding[1] + self.padding[3]
+
+    @cached_property
+    def vertical_padding(self):
+        return self.padding[0] + self.padding[2]
+
     def __lt__(self, img):
         """Use maxside, width, height or area as ordering algorithm.
 
@@ -495,6 +588,21 @@ class Sprite(object):
         self.cachebuster_hash = ''
 
         self.config = manager.config.extend(get_file_config(self.path))
+
+        # Build the set of ratios this sprite needs.
+        ratios = self.config.ratios.split(',')
+        self.ratios = set([float(r.strip()) for r in ratios if r.strip()])
+
+        # If the retina shortcut is in use add 2.0 as a required ratio.
+        if self.config.retina:
+            self.ratios.add(2.0)
+
+        # Always add 1.0 as a required ratio
+        self.ratios.add(1.0)
+
+        # Create a sorted list of ratios
+        self.ratios = sorted(self.ratios)
+
         self.process()
 
     def process(self):
@@ -545,15 +653,10 @@ class Sprite(object):
 
         return sorted(images, reverse=self.config.ordering[0] != '-')
 
-    def save_image(self):
-        """Create the image file for this sprite."""
-        self.manager.log("Creating '%s' image file..." % self.name)
-
-        sprite_output_path = self.manager.output_path('img')
-
-        # Search for the max x and y (Necessary to generate the canvas).
+    @cached_property
+    def canvas_size(self):
+        """Return the width and height for this sprite canvas"""
         width = height = 0
-
         for image in self.images:
             x = image.x + image.absolute_width
             y = image.y + image.absolute_height
@@ -561,31 +664,37 @@ class Sprite(object):
                 width = x
             if height < y:
                 height = y
+        return round_up(width), round_up(height)
+
+    def save_image(self):
+        """Create the image file for this sprite."""
+        self.manager.log("Creating '%s' image file..." % self.name)
 
         # Create the sprite canvas
+        width, height = self.canvas_size
         canvas = PImage.new('RGBA', (width, height), (0, 0, 0, 0))
 
         # Paste the images inside the canvas
         margin = int(self.config.margin)
         for image in self.images:
-            canvas.paste(image.image, (image.x + image.padding[3] + margin,
-                                       image.y + image.padding[0] + margin))
+            canvas.paste(image.image,
+                (round_up(image.x + (image.padding[3] + margin) * self.max_ratio),
+                 round_up(image.y + (image.padding[0] + margin) * self.max_ratio)))
 
         if self.config.cachebuster or self.config.cachebuster_filename:
             self.cachebuster_hash = hashlib.sha1(canvas.tostring()
                                                 ).hexdigest()[:6]
 
-        # Save png
-        sprite_filename = '%s.png' % self.filename
-        sprite_image_path = os.path.join(sprite_output_path, sprite_filename)
-
-        args, kwargs = [sprite_image_path], dict(optimize=True)
+        # Customize how the png is going to be saved
+        kwargs = dict(optimize=True)
 
         if self.config.png8:
             # Get the alpha band
             alpha = canvas.split()[-1]
             canvas = canvas.convert('RGB'
-                        ).convert('P', palette=PImage.ADAPTIVE, colors=255)
+                        ).convert('P',
+                                  palette=PImage.ADAPTIVE,
+                                  colors=255)
 
             # Set all pixel values below 128 to 255, and the rest to 0
             mask = PImage.eval(alpha, lambda a: 255 if a <= 128 else 0)
@@ -594,19 +703,36 @@ class Sprite(object):
             canvas.paste(255, mask)
             kwargs.update({'transparency': 255})
 
-        save = lambda: canvas.save(*args, **kwargs)
-        save()
+        # Loop all over the ratios and save one image for each one
+        for ratio in self.ratios:
 
-        if self.config.optipng:
-            command = ["%s %s" % (self.config.optipngpath,
-                                  sprite_image_path)]
+            sprite_image_path = self.image_path(ratio)
 
-            error = subprocess.call(command, shell=True, stdin=subprocess.PIPE,
-                            stdout=subprocess.PIPE)
-            if error:
-                self.manager.log("Error: optipng has failed, reverting to "
-                                 "the original file.")
-                save()
+            # If this canvas isn't the biggest one scale it using the ratio
+            if self.max_ratio != ratio:
+                reduced_canvas = canvas.resize(
+                    (round_up((width / self.max_ratio) * ratio),
+                     round_up((height / self.max_ratio) * ratio)),
+                    PImage.ANTIALIAS)
+            else:
+                reduced_canvas = canvas
+
+            save = lambda: reduced_canvas.save(sprite_image_path, **kwargs)
+            save()
+
+            # Optimize the image using optipng, if for some reason, it fails
+            # rollback to the original one.
+            if self.config.optipng:
+                command = ["%s %s" % (self.config.optipngpath,
+                                      sprite_image_path)]
+                error = subprocess.call(command,
+                                        shell=True,
+                                        stdin=subprocess.PIPE,
+                                        stdout=subprocess.PIPE)
+                if error:
+                    self.manager.log("Error: optipng has failed, reverting to "
+                                     "the original file.")
+                    save()
 
     def save_css(self):
         """Create the CSS or LESS file for this sprite."""
@@ -629,24 +755,43 @@ class Sprite(object):
         # add the global style for all the sprites for less bloat
         template = self.config.global_template.decode('unicode-escape')
         css_file.write(template % {'all_classes': class_names,
-                                   'sprite_url': self.image_url})
+                                   'sprite_url': self.image_url(),
+                                   'namespace': self.namespace})
 
         # compile one template for each file
         margin = int(self.config.margin)
+
         for image in self.images:
 
-            x = '%spx' % ((image.x * -1 if image.x else 0) - margin)
-            y = '%spx' % ((image.y * -1 if image.y else 0) - margin)
-            height = '%spx' % (image.absolute_height - (margin * 2))
-            width = '%spx' % (image.absolute_width - (margin * 2))
+            x = '%spx' % round_up((image.x * -1 - margin * self.max_ratio) / self.max_ratio)
+            y = '%spx' % round_up((image.y * -1 - margin * self.max_ratio) / self.max_ratio)
+
+            height = '%spx' % round_up((image.height / self.max_ratio) + image.vertical_padding)
+            width = '%spx' % round_up((image.width / self.max_ratio) + image.horizontal_padding)
 
             template = self.config.each_template.decode('unicode-escape')
             css_file.write(template % {'class_name': '.%s' % image.class_name,
-                                       'sprite_url': self.image_url,
+                                       'sprite_url': self.image_url(),
                                        'height': height,
                                        'width': width,
                                        'y': y,
                                        'x': x})
+
+        # If we have some additional ratio, we need to add one media query
+        # for each one.
+        if len(self.ratios) > 1:
+            canvas_size = zip(('width', 'height'),
+                              map(lambda s: '%spx' % int(s / self.max_ratio),
+                                  self.canvas_size))
+
+            for ratio in self.ratios:
+                if ratio != 1:
+                    data = dict(ratio=ratio,
+                                ratio_fraction=nearest_fration(ratio),
+                                sprite_url=self.image_url(ratio),
+                                all_classes=class_names,
+                                **dict(canvas_size))
+                    css_file.write(self.config.ratio_template % data)
         css_file.close()
 
     def save_html(self):
@@ -677,7 +822,7 @@ class Sprite(object):
                                          'version': __version__})
         html_file.close()
 
-    @property
+    @cached_property
     def namespace(self):
         """Return the namespace for this sprite."""
 
@@ -693,23 +838,44 @@ class Sprite(object):
             separator = ''
         return separator.join(namespace)
 
-    @property
+    @cached_property
     def filename(self):
         """Return the desired filename for files generated by this sprite."""
         if self.config.cachebuster_filename:
             return '%s_%s' % (self.name, self.cachebuster_hash)
         return self.name
 
-    @property
-    def image_path(self):
-        """Return the output path for the image file."""
-        return os.path.join(self.manager.output_path('img'),
-                            '%s.png' % self.filename)
+    def image_path(self, ratio=1):
+        reference = self.__get_reference(ratio)
+        """Return the output path for the image file.
 
-    @property
-    def image_url(self):
-        """Return the sprite image url."""
-        url = os.path.relpath(self.image_path, self.manager.output_path('css'))
+        :param ratio: Ratio.
+        """
+        return os.path.join(self.manager.output_path('img'),
+                            '%s%s.png' % (self.filename, reference))
+
+    def __get_reference(self, ratio):
+        """ Return the reference @Nx for this ratio.
+
+        :param ratio: Ratio.
+        """
+        reference = '@%.1fx' % ratio if int(ratio) != ratio else '@%ix' % ratio
+        if reference == '@1x':
+            reference = ''
+        return reference
+
+    @cached_property
+    def max_ratio(self):
+        """ Return the maximum ratio """
+        return max(self.ratios)
+
+    def image_url(self, ratio=1):
+        """Return the sprite image url.
+
+        :param ratio: Ratio.
+        """
+        url = os.path.relpath(self.image_path(ratio),
+                              self.manager.output_path('css'))
 
         if self.config.url:
             url = os.path.join(self.config.url, '%s.png' % self.filename)
@@ -738,6 +904,7 @@ class ConfigManager(object):
         self.defaults = kwargs.get('defaults', {})
         self.priority = kwargs.get('priority', {})
         self.sources = list(args)
+        self._cache = {}
 
     def extend(self, config):
         """Return a new :class:`~ConfigManager` instance with this new config
@@ -756,12 +923,21 @@ class ConfigManager(object):
 
         :param name: Configuration property name.
         """
+        if name in self._cache:
+            return self._cache[name]
+
         sources = [self.priority] + self.sources
+
         for source in sources:
             value = source.get(name)
             if value is not None:
-                return value
-        return self.defaults.get(name)
+                self._cache[name] = value
+                break
+
+        if name not in self._cache:
+            self._cache[name] = self.defaults.get(name)
+
+        return self._cache[name]
 
 
 class BaseManager(object):
@@ -814,7 +990,10 @@ class BaseManager(object):
         return sprite_output_path
 
     def log(self, message):
-        """Print the message if necessary."""
+        """Print the message if necessary.
+
+        :param message: Message to log.
+        """
         if not self.config.quiet:
             print(message)
 
@@ -881,13 +1060,19 @@ class SimpleSpriteManager(BaseManager):
 
 
 class WatchManager(object):
+    """ Watch a path for changes. """
 
     def __init__(self, path, action):
+        """
+        :param path: Path to watch.
+        :param action: Action to run when a change happens.
+        """
         self.action = action
         self.path = path
         self.last_hash = None
 
     def run(self):
+        """ Start watching the path for changes """
         signal.signal(signal.SIGINT, self.signal_handler)
 
         while True:
@@ -902,10 +1087,14 @@ class WatchManager(object):
                 time.sleep(0.2)
 
     def signal_handler(self, signal, frame):
+        """ Gracefully close the app if Ctrl+C is pressed."""
         print 'You pressed Ctrl+C!'
         sys.exit(0)
 
     def generate_hash(self):
+        """ Return a hash of files and modification times to determine if a
+        change has occourred."""
+
         hash_list = []
         for root, dirs, files in os.walk(self.path):
             for f in sorted([f for f in files if not f.startswith('.')]):
@@ -993,91 +1182,95 @@ PImage.Image.load = patched_load
 def main():
     parser = OptionParser(usage=("usage: %prog [options] source_dir [<output> "
                                  "| --css=<dir> --img=<dir>]"))
-    parser.add_option("--project", action="store_true",
-                  dest="project", help="generate sprites for multiple folders")
+    parser.add_option("--project", action="store_true", dest="project",
+            help="generate sprites for multiple folders")
     parser.add_option("-c", "--crop", dest="crop", action='store_true',
-                help="crop images removing unnecessary transparent margins")
+            help="crop images removing unnecessary transparent margins")
     parser.add_option("-l", "--less", dest="less", action='store_true',
-                help="generate output stylesheets as .less instead of .css")
-    parser.add_option("-u", "--url", dest="url", default=None,
-                      help="prepend this url to the sprites filename")
+            help="generate output stylesheets as .less instead of .css")
+    parser.add_option("-u", "--url", dest="url",
+            help="prepend this url to the sprites filename")
     parser.add_option("-q", "--quiet", dest="quiet", action='store_true',
-                      help="suppress all normal output")
-    parser.add_option("-p", "--padding", dest="padding", default=None,
-                      help="force this padding in all images")
+            help="suppress all normal output")
+    parser.add_option("-p", "--padding", dest="padding",
+            help="force this padding in all images")
+    parser.add_option("--ratios", dest="ratios",
+            help="Create sprites based on these ratios")
+    parser.add_option("--retina", dest="retina", action='store_true',
+            help="Shortcut for --ratios=2,1")
     parser.add_option("-w", "--watch", dest="watch", default=False,
-                      action='store_true',
-                      help=("Watch the source folder for changes and rebuild "
-                            "when new files appear, disappear or change."))
+            action='store_true',
+            help=("Watch the source folder for changes and rebuild "
+                  "when new files appear, disappear or change."))
     parser.add_option("-v", "--version", action="store_true", dest="version",
-                      help="show program's version number and exit")
+            help="show program's version number and exit")
 
     group = OptionGroup(parser, "Output Options")
-    group.add_option("--css", dest="css_dir", default='',
-                    help="output directory for css files", metavar='DIR')
+    group.add_option("--css", dest="css_dir", default='', metavar='DIR',
+            help="output directory for css files")
     group.add_option("--img", dest="img_dir", default='', metavar='DIR',
-                    help="output directory for img files")
-    group.add_option("--html", dest="html", default=None, action="store_true",
-                    help=("generate test html file using the sprite "
-                          "image and CSS."))
+            help="output directory for img files")
+    group.add_option("--html", dest="html", action="store_true",
+            help="generate test html file using the sprite image and CSS.")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Advanced Options")
-    group.add_option("-a", "--algorithm", dest="algorithm", default=None,
-                    help=("allocation algorithm: square, vertical, horizontal "
-                          "(default: square)"), metavar='NAME')
-    group.add_option("--ordering", dest="ordering", default=None,
-                    help=("ordering criteria: maxside, width, height or "
-                          "area (default: maxside)"), metavar='NAME')
-    group.add_option("--margin", dest="margin", default=None,
-                      help="force this margin in all images", type=int)
-    group.add_option("--namespace", dest="namespace",  default=None,
-                      help="namespace for all css classes (default: sprite)")
+    group.add_option("-a", "--algorithm", dest="algorithm", metavar='NAME',
+            help=("allocation algorithm: square, vertical, horizontal "
+                  "(default: square)"), )
+    group.add_option("--ordering", dest="ordering", metavar='NAME',
+            help=("ordering criteria: maxside, width, height or "
+                  "area (default: maxside)"), )
+    group.add_option("--margin", dest="margin", type=int,
+            help="force this margin in all images")
+    group.add_option("--namespace", dest="namespace",
+            help="namespace for all css classes (default: sprite)")
     group.add_option("--png8", action="store_true", dest="png8",
-                      help=("the output image format will be png8 "
-                            "instead of png32"))
-    group.add_option("--ignore-filename-paddings",
-                      dest="ignore_filename_paddings", action='store_true',
-                      help="ignore filename paddings")
+            help="the output image format will be png8 instead of png32")
+    group.add_option("--ignore-filename-paddings", action='store_true',
+            dest="ignore_filename_paddings", help="ignore filename paddings")
+    group.add_option("--debug", dest="debug", action='store_true',
+            help="don't catch unexpected errors and let glue fail hardly")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Output CSS Template Options")
-    group.add_option("--separator", dest="separator",
-                    help=("Customize the separator used to join CSS class "
-                          "names. If you want to use camelCase use "
-                          "'camelcase' as separator."),
-                    metavar='SEPARATOR')
+    group.add_option("--separator", dest="separator", metavar='SEPARATOR',
+            help=("Customize the separator used to join CSS class "
+                  "names. If you want to use camelCase use "
+                  "'camelcase' as separator."))
     group.add_option("--global-template", dest="global_template",
-                    help=("Customize the global section of the output CSS."
-                          "This section will be added only once for each "
-                          "sprite."),
-                    metavar='TEMPLATE')
-
+            metavar='TEMPLATE',
+            help=("Customize the global section of the output CSS."
+                  "This section will be added only once for each "
+                  "sprite."))
     group.add_option("--each-template", dest="each_template",
-                    help=("Customize each image output CSS."
-                          "This section will be added once for each "
-                          "image inside the sprite."),
-                    metavar='TEMPLATE')
+            metavar='TEMPLATE',
+            help=("Customize each image output CSS."
+                  "This section will be added once for each "
+                  "image inside the sprite."))
+    group.add_option("--ratio-template", dest="ratio_template",
+            metavar='TEMPLATE',
+            help=("Customize ratios CSS media queries template."
+                  "This section will be added once for each "
+                  "ratio different than 1."))
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Optipng Options",
-                      "You need to install optipng before using these options")
+                "You need to install optipng before using these options")
     group.add_option("--optipng", dest="optipng", action='store_true',
-                help="postprocess images using optipng")
-    group.add_option("--optipngpath", dest="optipngpath", default='optipng',
-                    help="path to optipng (default: optipng)", metavar='PATH')
+            help="postprocess images using optipng")
+    group.add_option("--optipngpath", dest="optipngpath",
+            help="path to optipng (default: optipng)", metavar='PATH')
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Browser Cache Invalidation Options")
-    group.add_option("--cachebuster", dest="cachebuster",
-                    action='store_true',
-                    help=("use the sprite's sha1 first 6 characters as a "
-                          "queryarg everytime that file is referred from "
-                          "the css"))
+    group.add_option("--cachebuster", dest="cachebuster", action='store_true',
+            help=("use the sprite's sha1 first 6 characters as a "
+                  "queryarg everytime that file is referred from the css"))
     group.add_option("--cachebuster-filename", dest="cachebuster_filename",
-                    action='store_true',
-                    help=("append the sprite's sha first 6 characters "
-                          "to the otput filename"))
+            action='store_true',
+            help=("append the sprite's sha first 6 characters "
+                  "to the otput filename"))
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args()
@@ -1156,6 +1349,15 @@ def main():
                           "images.\n") % e.args[0])
         sys.exit(e.error_code)
     except Exception:
+        if config.debug:
+            import platform
+            sys.stderr.write("Glue version: %s\n" % __version__)
+            sys.stderr.write("PIL version: %s\n" % PImage.VERSION)
+            sys.stderr.write("Platform: %s\n" % platform.platform())
+            sys.stderr.write("Config: %s\n" % config.sources)
+            sys.stderr.write("Args: %s\n" % sys.argv)
+            sys.stderr.write("\n")
+            raise
         sys.stderr.write("Error: Unknown Error.\n")
         sys.exit(1)
 
